@@ -1,10 +1,15 @@
+
+from typing import Callable, Tuple
+
 import torch
-from typing import Tuple, Callable
 from fastai.torch_core import Module
 
-from fastsom.core import expanded, ifnone, index_tensor
-from .neighborhood import neigh_gauss, neigh_diff_standard
+from fastsom.core import expanded, index_tensor
+from fastsom.core.tensor import idxs_1d_to_2d, idxs_2d_to_1d
+
+from ..log import has_logger
 from .distance import pdist
+from .neighborhood import neigh_diff_standard, neigh_gauss
 
 
 class Som(Module):
@@ -24,11 +29,11 @@ class Som(Module):
     """
 
     def __init__(
-            self,
-            size: Tuple[int, int, int],
-            dist_fn: Callable = pdist,
-            neigh_fn: Callable = neigh_gauss,
-            neigh_diff_fn: Callable = neigh_diff_standard,
+        self,
+        size: Tuple[int, int, int],
+        dist_fn: Callable = pdist,
+        neigh_fn: Callable = neigh_gauss,
+        neigh_diff_fn: Callable = neigh_diff_standard,
     ) -> None:
         super().__init__()
         self.size = size
@@ -56,14 +61,16 @@ class Som(Module):
         xb : torch.Tensor
             The batch data
         """
-        self.to_device(device=xb.device)
-        n_features = xb.shape[-1]
-        distances = self.distance(xb, self.weights.view(-1, n_features))
-        bmus = self.find_bmus(distances)
-        # save batch data
-        self._recorder['xb'] = xb.clone()
-        self._recorder['bmus'] = bmus
-
+        with torch.no_grad():
+            self.to_device(device=xb.device)
+            n_features = xb.shape[-1]
+            # self.logger.debug(f"xb: {xb.shape}, weights: {self.weights.view(-1, n_features).shape}")
+            distances = self.distance(xb, self.weights.view(-1, n_features))
+            bmus = self.find_bmus(distances)
+            # self.logger.debug(f"bmus: {bmus.shape}")
+            # save batch data
+            self._recorder["xb"] = xb.clone()
+            self._recorder["bmus"] = bmus
         return bmus
 
     def distance(self, a: torch.Tensor, b: torch.Tensor) -> torch.Tensor:
@@ -89,12 +96,15 @@ class Som(Module):
          2. Calculate neighbourhood scaling on index distances
          3. Update weights
         """
-        xb, bmus = self._recorder['xb'], self._recorder['bmus']
-        batch_size = xb.shape[0]
-        n_features = xb.shape[-1]
-        elementwise_diffs = expanded(xb, self.weights.view(-1, n_features), lambda a, b: a - b).view(batch_size, self.size[0], self.size[1], n_features)
-        neighbourhood_mults = self.neighborhood(bmus, self.sigma)
-        self.weights += (self.alpha * neighbourhood_mults * elementwise_diffs / batch_size).sum(0)
+        with torch.no_grad():
+            xb, bmus = self._recorder["xb"], self._recorder["bmus"]
+            batch_size = xb.shape[0]
+            n_features = xb.shape[-1]
+            # diffs shape: [bs, row_sz, col_sz, n_features]
+            elementwise_diffs = expanded(xb, self.weights.view(-1, n_features), lambda a, b: a - b).view(batch_size, self.size[0], self.size[1], n_features)
+            # neigh shape: [bs, row_sz, col_zs, 1]
+            neighbourhood_mults = self.neighborhood(bmus, self.sigma)
+            self.weights += (self.alpha * neighbourhood_mults * elementwise_diffs).sum(0) / batch_size
 
     def find_bmus(self, distances: torch.Tensor) -> torch.Tensor:
         """
@@ -105,9 +115,11 @@ class Som(Module):
         distances : torch.Tensor
             The data-to-codebook distances.
         """
+        # distances shape: [bs, codebook_size]
+        # argmin shape   : [bs, 1]
         min_idxs = distances.argmin(-1)
         # Distances are flattened, so we need to transform 1d indices into 2d map locations
-        return torch.stack([min_idxs / self.size[1], min_idxs % self.size[1]], dim=1)
+        return idxs_1d_to_2d(min_idxs, self.size[1]).to(distances.device)
 
     def neighborhood(self, bmus: torch.Tensor, sigma: torch.Tensor) -> torch.Tensor:
         """
@@ -135,12 +147,12 @@ class Som(Module):
         self.map_indices = self._to_device(self.map_indices, device=device)
 
     def _to_device(self, a: torch.Tensor, device: torch.device = None) -> torch.Tensor:
-        """Moves a tensor to the appropriate device"""
+        """Moves a tensor to the appropriate device."""
         if a.device != device:
             a = a.to(device=device)
         return a
 
     def __repr__(self):
-        return f'{self.__class__.__name__}(\n\
+        return f"{self.__class__.__name__}(\n\
             size={self.size[:-1]}, neuron_size={self.size[-1]}, alpha={self.alpha}, sigma={self.sigma}),\n\
-            dist_fn={self.dist_fn.__name__}, neigh_fn={self.neigh_fn.__name__}, neigh_diff_fn={self.neigh_diff_fn})'
+            dist_fn={self.dist_fn.__name__}, neigh_fn={self.neigh_fn.__name__}, neigh_diff_fn={self.neigh_diff_fn})"
